@@ -1,12 +1,10 @@
 ;;; document.el --- Use the bovinator to aid in generating documentation.
 
-;;; Copyright (C) 2000 Eric M. Ludlam
+;;; Copyright (C) 2000, 2001 Eric M. Ludlam
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 ;; Keywords: doc
-;; X-RCS: $Id: document.el,v 1.5 2001/02/20 03:23:43 youngs Exp $
-
-;; This file is not part of GNU Emacs.
+;; X-RCS: $Id: document.el,v 1.6 2001/08/16 03:37:27 andyp Exp $
 
 ;; Semantic is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -33,6 +31,10 @@
 ;;   Document then provides some rules for creating English Text based
 ;; on the name of a given function, it's return type, or variable
 ;; type.  It also has output rules for texinfo, or comments.
+;;
+;; NOTE: Some of the user level commands in document.el dealing with
+;; texinfo files have been obsoleted commands in semantic-texi, which
+;; can not insert foriegn tokens.
 
 (require 'sformat)
 (require 'document-vars)
@@ -99,7 +101,9 @@ When non-nil, query for a new documentation file."
 	(error "No file found for your documentation"))
     (set-buffer (marker-buffer document-current-output-file))
     (goto-char document-current-output-file)
+    (insert "\n")
     (document-insert-texinfo cdi cdib)
+    (insert "\n")
     (setq document-current-output-file (point-marker))
     ))
 
@@ -114,13 +118,16 @@ When non-nil, query for a new documentation file."
 ;;
 (defun document-insert-texinfo (nonterm buffer)
   "Insert texinfo documentation about NONTERM from BUFFER."
-  (insert "\n")
   (let ((tt (semantic-token-token nonterm)))
     (insert "@"
 	    (cond ((eq tt 'variable)
-		   "defvar")
+		   (if (semantic-token-extra-spec nonterm 'user-visible)
+		       "deffn Option"
+		     "defvar"))
 		  ((eq tt 'function)
-		   "defun")
+		   (if (semantic-token-extra-spec nonterm 'user-visible)
+		       "deffn Command"
+		     "defun"))
 		  ((eq tt 'type)
 		   "deftype")
 		  (t (error "Don't know how to document that")))
@@ -141,12 +148,16 @@ When non-nil, query for a new documentation file."
 	     (document-generate-documentation nonterm buffer)))
     (insert "\n@end "
 	    (cond ((eq tt 'variable)
-		   "defvar")
+		   (if (semantic-token-extra-spec nonterm 'user-visible)
+		       "deffn"
+		     "defvar"))
 		  ((eq tt 'function)
-		   "defun")
+		   (if (semantic-token-extra-spec nonterm 'user-visible)
+		       "deffn"
+		     "defun"))
 		  ((eq tt 'type)
 		   "deftype"))
-	    "\n")))
+	    )))
 
 (defun document-insert-defun-comment (nonterm buffer)
   "Insert mode-comment documentation about NONTERM from BUFFER."
@@ -266,18 +277,20 @@ Adds the comment line PREFIX to each line."
 ;;
 (defun document-generate-documentation (nonterm buffer)
   "Return a plain string documenting NONTERM from BUFFER."
-  (let ((doc ;; Second, does this thing have docs in the source buffer which
-	 ;; an override method might be able to find?
-	 (semantic-find-documentation nonterm)
-	 ))
-    (if (not doc)
-	(document-generate-new-documentation nonterm buffer)
-      ;; Ok, now lets see what sort of formatting there might be,
-      ;; and see about removing some of it.. (Tables of arguments,
-      ;; and that sort of thing.)
-      nil
-      ;; Return the string.
-      doc)))
+  (save-excursion
+    (set-buffer buffer)
+    (let ((doc ;; Second, does this thing have docs in the source buffer which
+	   ;; an override method might be able to find?
+	   (semantic-find-documentation nonterm)
+	   ))
+      (if (not doc)
+	  (document-generate-new-documentation nonterm buffer)
+	;; Ok, now lets see what sort of formatting there might be,
+	;; and see about removing some of it.. (Tables of arguments,
+	;; and that sort of thing.)
+	nil
+	;; Return the string.
+	doc))))
 
 (defun document-generate-new-documentation (nonterm buffer)
   "Look at elements of NONTERM in BUFFER to make documentation.
@@ -764,7 +777,7 @@ This is to take advantage of TeXinfo's markup symbols."
   ;; run the elisp version through also.
   (let ((case-fold-search nil))
     (while (string-match
-	    "\\(^\\|[^{]\\)\\<\\([A-Z0-9_]+\\)\\>\\($\\|[^}]\\)"
+	    "\\(^\\|[^{]\\)\\<\\([A-Z0-9_-]+\\)\\>\\($\\|[^}]\\)"
 	    string)
       (setq string (concat (substring string 0 (match-beginning 2))
 			   "@var{"
@@ -784,6 +797,7 @@ that class.
  `variable' => @code{variable}
  `class'    => @code{class} @xref{class}
  `unknown'  => @code{unknonwn}
+ \"text\"     => ``text''
  'quoteme   => @code{quoteme}
  non-nil    => non-@code{nil}
  t          => @code{t}
@@ -808,6 +822,8 @@ that class.
     (setq string (replace-match "@code{\\2}" t nil string 2)))
   (while (string-match "\\( \\|^\\)\\(\\(\\(C-\\|M-\\|S-\\)+\\([^ \t\n]\\|RET\\|SPC\\|TAB\\)\\)\\|\\(RET\\|SPC\\|TAB\\)\\)\\( \\|$\\)" string)
     (setq string (replace-match "@kbd{\\2}" t nil string 2)))
+  (while (string-match "\"\\(.+\\)\"" string)
+    (setq string (replace-match "``\\1''" t nil string 0)))
   string)
 
 ;;; Buffer finding and managing
